# regex-cache [![NPM version](https://badge.fury.io/js/regex-cache.svg)](http://badge.fury.io/js/regex-cache)  [![Build Status](https://travis-ci.org/jonschlinkert/regex-cache.svg)](https://travis-ci.org/jonschlinkert/regex-cache) 

> Memoize the results of a call to the RegExp constructor, avoiding repetitious runtime compilation of the same string and options, resulting in dramatic speed improvements.


## Install with [npm](npmjs.org)

```bash
npm i regex-cache --save
```

## Usage

If the lib returns a function, you can wrap it like this:

```js
var cache = require('regex-cache');
var regex = cache(require('some-regex-lib'));
```

Or, if the lib (or your own function) returns a regex, you can wrap it with a function first to achieve the same results.

**Recommendations**

* **do not** use this if you are passing options 
* **do** use this if you are passing options and the logic to create the regex is extensive
* **do** use this if you're not passing options and the lib returns a function (e.g. `re().test(str)`)


### Example benchmarks

[Performance results](#benchmarks) for a random regex lib, [mentions-regex], with and without regex-cache, and no options passed:

```bash
with regex-cache x 11,256,663 ops/sec ±0.59% (97 runs sampled)
without regex-cache x 1,812,916 ops/sec ±1.66% (24 runs sampled)
```

## What it does

If you're using `new RegExp('foo')` instead of a regex literal, it's probably because you need to dyamically generate a regex based on user options or some other potentially changing factors. 

regex-cache is a simple way to cache the results of a previous call to the RegExp constructor to avoid unnecessary runtime compilation when both the string and options being passed have not changed.

Using the RegExp constructor offers a lot of flexibility, but the runtime compilation comes at a price - it's slow. Not specifically because of the call to the RegExp constructor, but because you have to build up the string to create the regex based on various inputs. 

## Run tests

Install dev dependencies:

```bash
npm i -d && npm test
```

## Run benchmarks

Install dev dependencies:

```bash
npm i -d && npm run benchmarks
```

## Contributing
Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](https://github.com/jonschlinkert/regex-cache/issues)

## Author

**Jon Schlinkert**
 
+ [github/jonschlinkert](https://github.com/jonschlinkert)
+ [twitter/jonschlinkert](http://twitter.com/jonschlinkert) 

## License
Copyright (c) 2015 Jon Schlinkert  
Released under the MIT license

***

_This file was generated by [verb](https://github.com/assemble/verb) on February 09, 2015._

[mentions-regex]: https://github.com/regexps/mentions-regex
